@@ -23,7 +23,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: ssh-rsa.c,v 1.18 2002/04/02 20:11:38 markus Exp $");
+RCSID("$OpenBSD: ssh-rsa.c,v 1.20 2002/06/10 16:53:06 stevesk Exp $");
 
 #include <openssl/evp.h>
 #include <openssl/err.h>
@@ -115,7 +115,7 @@ ssh_rsa_verify(
 	EVP_MD_CTX md;
 	char *ktype;
 	u_char digest[EVP_MAX_MD_SIZE], *sigblob;
-	u_int len, dlen;
+	u_int len, dlen, modlen;
 	int rlen, ret, nid;
 
 	if (key == NULL || key->type != KEY_RSA || key->rsa == NULL) {
@@ -123,8 +123,8 @@ ssh_rsa_verify(
 		return -1;
 	}
 	if (BN_num_bits(key->rsa->n) < SSH_RSA_MINIMUM_MODULUS_SIZE) {
-		error("ssh_rsa_verify: n too small: %d bits",
-		    BN_num_bits(key->rsa->n));
+		error("ssh_rsa_verify: RSA modulus too small: %d < minimum %d bits",
+		    BN_num_bits(key->rsa->n), SSH_RSA_MINIMUM_MODULUS_SIZE);
 		return -1;
 	}
 	buffer_init(&b);
@@ -145,6 +145,21 @@ ssh_rsa_verify(
 		xfree(sigblob);
 		return -1;
 	}
+	/* RSA_verify expects a signature of RSA_size */
+	modlen = RSA_size(key->rsa);
+	if (len > modlen) {
+		error("ssh_rsa_verify: len %d > modlen %d", len, modlen);
+		xfree(sigblob);
+		return -1;
+	} else if (len < modlen) {
+		int diff = modlen - len;
+		debug("ssh_rsa_verify: add padding: modlen %d > len %d",
+		    modlen, len);
+		sigblob = xrealloc(sigblob, modlen);
+		memmove(sigblob + diff, sigblob, len);
+		memset(sigblob, 0, diff);
+		len = modlen;
+	}
 	nid = (datafellows & SSH_BUG_RSASIGMD5) ? NID_md5 : NID_sha1;
 	if ((evp_md = EVP_get_digestbynid(nid)) == NULL) {
 		error("ssh_rsa_verify: EVP_get_digestbynid %d failed", nid);
